@@ -24,6 +24,7 @@
 #include "lx2_ops.h"
 #include "errors.h"
 
+#include <string.h>
 #include <stdarg.h>
 #include <libxml/globals.h>
 #include <libxml/parser.h>
@@ -40,13 +41,13 @@
 #endif // WITHOUT_LIBEXSLT
 #endif // WITHOUT_LIBXSLT
 
-map<string, fileInfo> loadedFiles;
+std::map<std::string, fileInfo> loadedFiles;
 
 #ifndef WITHOUT_LIBXSLT
-map<string, xsltStylesheetPtr> parsedXsltFiles;
+std::map<std::string, xsltStylesheetPtr> parsedXsltFiles;
 #endif // WITHOUT_LIBXSLT
 
-// #define WITH_PARSERCTX
+#define WITH_PARSERCTX
 
 #ifdef WITH_PARSERCTX
 // The parser Context
@@ -106,7 +107,7 @@ void xmlFinalize(const struct globalOptions & options)
 
 
 /// Get the XML File tree
-xmlNodePtr getXmlFile(const string & alias, const struct globalOptions & options)
+xmlNodePtr getXmlFile(const std::string & alias, const struct globalOptions & options)
 {
     if (loadedFiles.find(alias) != loadedFiles.end())
     {
@@ -130,7 +131,7 @@ xmlNodePtr getXmlFile(const string & alias, const struct globalOptions & options
 
 #ifndef WITHOUT_LIBXSLT
 /// Get the XSLT File
-xsltStylesheetPtr getXsltFile(const string & alias, const struct globalOptions & options)
+xsltStylesheetPtr getXsltFile(const std::string & alias, const struct globalOptions & options)
 {
     struct globalOptions localOptions;
     localOptions = options;
@@ -156,7 +157,7 @@ xsltStylesheetPtr getXsltFile(const string & alias, const struct globalOptions &
 
 
 /// Load an XML File according to provided options.
-int loadXmlFile(string filename, string alias, const struct globalOptions & options)
+int loadXmlFile(std::string filename, std::string alias, const struct globalOptions & options)
 {
     fileInfo fi;
     xmlDocPtr doc;
@@ -165,7 +166,7 @@ int loadXmlFile(string filename, string alias, const struct globalOptions & opti
 #ifdef WITH_PARSERCTX
     doc = xmlCtxtReadFile(ctxt, 
                           filename.c_str(), 
-                          NULL, 
+                          (strcmp((const char *)options.encoding.c_str(),"none")!=0)?(const char *)options.encoding.c_str():NULL, 
                           (options.cleanText)?XML_PARSE_NOBLANKS:0);
 #else
 	xmlKeepBlanksDefault((options.cleanText)?0:1);
@@ -202,7 +203,7 @@ int loadXmlFile(string filename, string alias, const struct globalOptions & opti
 }
 
 /// Save an XML File according to provided options.
-int saveXmlFile(string filename, string alias, const struct globalOptions & options)
+int saveXmlFile(std::string filename, std::string alias, const struct globalOptions & options)
 {
     xmlNodePtr node;
     node = getXmlFile(alias, options);
@@ -232,7 +233,7 @@ int saveXmlFile(string filename, string alias, const struct globalOptions & opti
 }
 
 /// Close the XML File 
-void closeXmlFile(string alias, const struct globalOptions & options)
+void closeXmlFile(std::string alias, const struct globalOptions & options)
 {
     if (loadedFiles.find(alias) != loadedFiles.end())
     {
@@ -262,7 +263,7 @@ void closeXmlFile(string alias, const struct globalOptions & options)
 /// Flush files
 void flushXmlFiles(const struct globalOptions & options)
 {
-    map<string, fileInfo>::iterator it;
+    std::map<std::string, fileInfo>::iterator it;
 #ifndef WITHOUT_LIBXSLT
 	// Free XML with xslt before all XML files (XSLT file is needed)
     for(it = loadedFiles.begin(); it != loadedFiles.end(); it++)
@@ -278,7 +279,7 @@ void flushXmlFiles(const struct globalOptions & options)
 }
 
 /// Diff Trees XML Files
-int diffXmlFiles(string beforeAlias, string afterAlias, string outputAlias, const struct globalOptions & options)
+int diffXmlFiles(std::string beforeAlias, std::string afterAlias, std::string outputAlias, const struct globalOptions & options)
 {
     int rc;
     xmlNodePtr beforeNode, afterNode, outputNode;
@@ -347,9 +348,9 @@ int diffXmlFiles(string beforeAlias, string afterAlias, string outputAlias, cons
 }
 
 /// Recalc an XML File according to provided options
-int recalcXmlFiles(string alias, const struct globalOptions & options)
+int recalcXmlFiles(std::string alias, const struct globalOptions & options)
 {
-    int rc;
+    int rc = -1;
     xmlNodePtr node;
     node = getXmlFile(alias, options);
     verbose(2, options.verboseLevel, "Recalculating %s...", alias.c_str());
@@ -363,7 +364,7 @@ int recalcXmlFiles(string alias, const struct globalOptions & options)
 }
 
 /// Delete nodes of the file
-int deleteNodes(const string & alias, const xmlstring & xpath, const struct globalOptions & options)
+int deleteNodes(const std::string & alias, const xmlstring & xpath, const struct globalOptions & options)
 {
     int i, nb;
     xmlNodePtr node, curNode;
@@ -379,7 +380,7 @@ int deleteNodes(const string & alias, const xmlstring & xpath, const struct glob
 	curNode = NULL;
 	if (node->doc != NULL) curNode = node->doc->children;
 	while ((curNode != NULL) && ((curNode->type != XML_ELEMENT_NODE))) curNode = curNode->next;
-	if (curNode != NULL) 
+	if ((curNode != NULL)  && (xpathCtx != NULL))
 	{
 		xpathCtx->namespaces = xmlGetNsList(node->doc, curNode);
 		xpathCtx->nsNr = 0;
@@ -390,41 +391,52 @@ int deleteNodes(const string & alias, const xmlstring & xpath, const struct glob
     xpathObj = xmlXPathEvalExpression(xpath.c_str(), xpathCtx);
     if (xpathObj != NULL)
     {
+		std::vector<xmlNodePtr> vnodelist;
         nb = ((xpathObj->nodesetval)?xpathObj->nodesetval->nodeNr:0);
         verbose(2, options.verboseLevel, "Deleting %s = %d nodes...", BAD_CAST xpath.c_str(), nb);
+		// Copy in std::vector
+        for(i=0;i<nb;i++) { vnodelist.push_back(xpathObj->nodesetval->nodeTab[i]); }
+		// Free XPath
+        xmlXPathFreeObject(xpathObj);
+		// Delete nodes
         for(i=0;i<nb;i++)
         {
-            xmlUnlinkNode(xpathObj->nodesetval->nodeTab[i]);
-            xmlFreeNode(xpathObj->nodesetval->nodeTab[i]);
+			if (vnodelist[i]) {
+				xmlUnlinkNode(vnodelist[i]);
+				xmlFreeNode(vnodelist[i]);
+			}
         }
         verbose(2, options.verboseLevel, " done.\n");
-        xmlXPathFreeObject(xpathObj);
         if (nb != 0) loadedFiles[alias].modified = true;
     }
+	
+	if ((xpathCtx) && (xpathCtx->namespaces)) xmlFree(xpathCtx->namespaces);
     xmlXPathFreeContext(xpathCtx); 
     return 0;
 }
 
 
 /// Duplicate Document
-int duplicateDocument(const string & src, const string & dest, const struct globalOptions & options)
+int duplicateDocument(const std::string & src, const std::string & dest, const struct globalOptions & options)
 {
     xmlNodePtr srcNode, destNode;
     srcNode = getXmlFile(src, options);
     if (srcNode == NULL) throwError(XD_Exception::XDE_DIFF_MEMORY_ERROR, "Unable to instanciate stuctures, probably due to a memory problem");
-    verbose(3, options.verboseLevel, "Duplicating %s to %d...", src.c_str(), dest.c_str());
-    destNode = (xmlNodePtr)xmlCopyDoc(srcNode->doc, 1);
-    verbose(3, options.verboseLevel, " done.\n");
-    loadedFiles[dest] = loadedFiles[src];
-    loadedFiles[dest].doc = destNode->doc;
-    loadedFiles[dest].filename = dest;
-    loadedFiles[dest].modified = true;
-    loadedFiles[dest].opened = true;
+	else {
+		verbose(3, options.verboseLevel, "Duplicating %s to %s...", src.c_str(), dest.c_str());
+		destNode = (xmlNodePtr)xmlCopyDoc(srcNode->doc, 1);
+		verbose(3, options.verboseLevel, " done.\n");
+		loadedFiles[dest] = loadedFiles[src];
+		loadedFiles[dest].doc = destNode->doc;
+		loadedFiles[dest].filename = dest;
+		loadedFiles[dest].modified = true;
+		loadedFiles[dest].opened = true;
+	}
     return 0;
 }
 
 #ifndef WITHOUT_LIBXSLT
-int applyStylesheet(const string & xslt, const string & src, const string & dest, const char ** params, const struct globalOptions & options)
+int applyStylesheet(const std::string & xslt, const std::string & src, const std::string & dest, const char ** params, const struct globalOptions & options)
 {
 	int ret = 0;
     xmlDocPtr doc, res;
@@ -444,8 +456,8 @@ int applyStylesheet(const string & xslt, const string & src, const string & dest
     res = xsltApplyStylesheetUser(xsltPtr, doc, params, NULL, NULL, ctxt);
 	ret = ctxt->state;
 	xsltFreeTransformContext(ctxt);
-	if (ret == XSLT_STATE_ERROR) { throwError(XD_Exception::XDE_XSLT_ERROR, "XSLT Transformation Error"); return -2; }
-	if (ret == XSLT_STATE_STOPPED) { throwError(XD_Exception::XDE_XSLT_STOPPED, "XSLT Transformation Stopped"); return -3; }
+	if (ret == XSLT_STATE_ERROR) { xsltFreeStylesheet(xsltPtr); xmlFreeDoc(res); throwError(XD_Exception::XDE_XSLT_ERROR, "XSLT Transformation Error"); return -2; }
+	if (ret == XSLT_STATE_STOPPED) {  xsltFreeStylesheet(xsltPtr); xmlFreeDoc(res); throwError(XD_Exception::XDE_XSLT_STOPPED, "XSLT Transformation Stopped"); return -3; }
 	//
     verbose(2, options.verboseLevel, " done.\n");
     if (src == dest)
@@ -478,6 +490,7 @@ void setDefaultXmldiffOptions(struct appCommand & opt)
     opt.diff_ns = BAD_CAST "http://www.via.ecp.fr/~remi/soft/xml/xmldiff";
     opt.diff_xmlns =  BAD_CAST "diff";
     opt.separator = BAD_CAST "|";
+    opt.encoding = BAD_CAST "none";
     opt.diffQualifiersList[0] = BAD_CAST "unknown";
     opt.diffQualifiersList[1] = BAD_CAST "added";
     opt.diffQualifiersList[2] = BAD_CAST "removed";
@@ -495,6 +508,7 @@ void setDefaultXmldiffOptions(struct appCommand & opt)
     opt.forceClean = false;
     opt.automaticSave = true;
     opt.optimizeMemory = false;
+	opt.specialNodesIds = true;
 #ifndef WITHOUT_LIBXSLT
 #ifndef WITHOUT_LIBEXSLT
     opt.useEXSLT = false;
